<?php
namespace SCM\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;

use Composer\Semver\Semver;
use Composer\Semver\Intervals;
use Composer\Semver\Constraint\Constraint;
use Composer\Semver\VersionParser;
use Composer\Semver\Comparator;

class Server extends \Zeedhi\Framework\Controller\Simple {

// 	const VERSION_STATUS = '3.4';
	
	protected $version;
	protected $frontVersionSupported;
	
	public function __construct(\Zeedhi\Framework\DependencyInjection\InstanceManager $instanceManager) {
	    $this->version = $instanceManager->getParameter('version');
	    $this->frontVersionSupported = $instanceManager->getParameter('frontVersionSupported');
	}

	public function Server(Request $request, Response $response) {
        try {
            // $response->addDataSet(new DataSet('connection', array('version' => $this->version)));
            $params = $request->getParameters();
            if (\Zeedhi\Framework\Util\Functions::arrayKeyExists('frontVersion', $params)) {
                $frontVersion = $params['frontVersion'];
                $status = '';
                $verify = Semver::satisfies($frontVersion, $this->frontVersionSupported);
                if ($verify === false) {
                    $versionParser = new VersionParser;
                    $versionParser->normalizeStability('stable');
                    $versionParser->normalize($this->version);
                    $constraint = $versionParser->parseConstraints($this->frontVersionSupported);
                    if (Comparator::greaterThanOrEqualTo($constraint->getLowerBound()->getVersion(), $frontVersion)) {
                        // versão do front menor que a menor versão permitida pela expressão - front precisa atualizar
                        $status = 'NEW_VERSION';
                    } else if (Comparator::greaterThanOrEqualTo($frontVersion, $constraint->getUpperBound()->getVersion())) {
                        // versão do front maior que a maior versão permitida pela expressão - back precisa atualizar
                        $status = 'SERVER_OUT';
                    } else $status = 'SERVER_OUT';
                } else if ($verify === true) {
                    $status = 'OK';
                }
                $response->addDataSet(new DataSet('connection', array('status' => $status)));
            } else throw new \Exception("Versão do aplicativo não informada!");
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function Exportacao(Request $request, Response $response) {
        try {
			$response->addDataSet(new DataSet('Exportacao', array()));
        } catch (\Exception $e) {
           $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
}