<?php
/**
 * Created by PhpStorm.
 * User: francismararaujo
 * Date: 23/10/14
 * Time: 17:07
 */

namespace SCM\Services;

use Doctrine\ORM\EntityManager;
use SCM\Util\CRUD;
use SCM\Traits\TQuery;

class UserService{

    use TQuery;

    protected $entityManager;
    protected $connection;
    protected $CRUD;

    const UPDATE_FILIAL_SELECIONADA = "UPDATE OPERADOR SET CDFILIALSELECIONADA = :CDSELECTEDBRANCH WHERE CDOPERADOR = :OPERATOR AND NRORG = :ORGANIZATION";
	const UPDATE_USER = "UPDATE OPERADOR SET NMOPERADOR = :NMOPERADOR, CDSENHAOPERWEB = :CDSENHAOPERWEB WHERE DSEMAILOPER = :DSEMAILOPER";
	const USER_BY_MAIL = "SELECT NRORG, CDOPERADOR, DSEMAILOPER, NMOPERADOR, IDOPERATIVO, CDSENHOPER, CDSENHAOPERWEB, NRESTRUTTRAB, NRORGTRAB FROM OPERADOR WHERE DSEMAILOPER = :DSEMAILOPER";
    const FILTER_FILIAIS = "SELECT F.NMFILIAL,
                                   F.CDEMPRESA,
                                   F.CDFILIAL,
                                   NVL(F.CDFILCENTPME, F.CDFILIAL) CDFILLANCA,
                                   F.NRORG,
                                   DECODE(F.CDFILCENTCMP, NULL, F.CDFILIAL, F.CDFILCENTCMP) AS CDFILCENTCMP,
                                   DECODE(F.IDFILCENTPME, 'S', F.CDFILIAL, 'N', F.CDFILCENTPME, NULL) AS CDFILCENTPME
                              FROM FILIOPER FO
                             INNER JOIN FILIAL F
                                ON FO.CDFILIAL   = F.CDFILIAL
                               AND FO.NRORG      = F.NRORG
                             WHERE FO.CDOPERADOR = :CDOPERADOR
                               AND (INSTR(:CDFILIAL, '_' || F.CDFILIAL || '_') <> 0)
                             ORDER BY F.CDFILIAL";

    const GET_FILIAIS = "SELECT F.NMFILIAL,
                            F.CDEMPRESA,
                            F.CDFILIAL,
                            NVL(F.CDFILCENTPME, F.CDFILIAL) CDFILLANCA,
                            F.CDFILIAL || ' - ' || F.NMFILIAL AS DESCRICAO,
                            F.NRORG,
                            DECODE(F.CDFILCENTCMP, NULL, F.CDFILIAL, F.CDFILCENTCMP) AS CDFILCENTCMP,
                            DECODE(F.IDFILCENTPME, 'S', F.CDFILIAL, 'N', F.CDFILCENTPME, NULL) AS CDFILCENTPME
                       FROM FILIOPER FO
                      INNER JOIN FILIAL F
                         ON FO.CDFILIAL = F.CDFILIAL
                        AND FO.NRORG = F.NRORG
                        AND F.CDFILIAL <> F.CDFILCENTPME
                         JOIN FILIAL C 
                            ON C.CDFILIAL = F.CDFILCENTPME
                            AND C.IDTPFILIALCENT = 'M'
                      WHERE FO.CDOPERADOR = :CDOPERADOR
                      AND ( INSTR(:CDFILIAL, F.CDFILIAL) <> 0 OR (:CDFILIAL = 'T' ) )
                      ORDER BY F.CDFILIAL";

    public function __construct(EntityManager $entityManager, CRUD $CRUD) {

        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
        $this->CRUD = $CRUD;
    }

    public function updateUser($row) {

        $NMOPERADOR = $row['NMOPERADOR'];
        $DSEMAILOPER = $row['DSEMAILOPER'];
        $CDSENHAATUAL = MD5($row['SENHA']);
        $CDSENHAOPERWEB = MD5($row['NOVASENHA']);

        $params = array(
            'DSEMAILOPER' => $DSEMAILOPER
        );
        $user = $this->CRUD->read(self::USER_BY_MAIL, $params);

        if(!$user) {
            throw new \Exception("Operador não está cadastrado.");
        }

        if(strlen($row['SENHA']) === 0){
            $CDSENHAATUAL = $CDSENHAOPERWEB = $user[0]['CDSENHAOPERWEB'];
        }

        if($user[0]['CDSENHAOPERWEB'] != $CDSENHAATUAL) {
            throw new \Exception("Senha inválida.");
        }

        $params = array(
            "NMOPERADOR" => $NMOPERADOR,
            "DSEMAILOPER" => $DSEMAILOPER,
            "CDSENHAOPERWEB" => $CDSENHAOPERWEB
        );

        $this->CRUD->update(self::UPDATE_USER, $params);
    }

    public function save($user) {

        $params = array(
            'CDSELECTEDBRANCH' => $user['CDFILIAL'],
            'OPERATOR' => $user['CDOPERADOR'],
            'ORGANIZATION' => $user['NRORG']
        );
        $this->CRUD->update(self::UPDATE_FILIAL_SELECIONADA, $params);
    }

    public function retornaFiliaisUsuario($operador, $cdfilial) {

        $params = array(
            'CDOPERADOR' => $operador,
            'CDFILIAL' => $cdfilial,
        );
        // return $this->CRUD->read(self::GET_FILIAIS, $params);
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, self::GET_FILIAIS, $params);
    }

    public function filtraFiliaisUsuario($operador, $filiais) {

        $params = array(
            'CDOPERADOR' => $operador,
            'CDFILIAL' => "_" . implode("_",$filiais) . "_"
        );
        return $this->CRUD->read(self::FILTER_FILIAIS, $params);
    }

    public function retornaUsuarioAtivo($dsemailoper, $nrorg) {
        $operador =  $this->entityManager->getRepository('SCM\Model\Complemento')->retornaUsuarioAtivo($dsemailoper, $nrorg);

        if ($operador[0] && $operador[0]['IDOPERATIVO'] == 'A') {
            return true;
        } else {
            return false;
        }
    }
    
    public function retornaOperadorAtivo($cdoperador, $nrorg) {
        $operador =  $this->entityManager->getRepository('SCM\Model\Complemento')->retornaOperadorAtivo($cdoperador, $nrorg);

        if ($operador[0] && $operador[0]['IDOPERATIVO'] == 'A') {
            return true;
        } else {
            return false;
        }
    }
}